/* rltm.c
 * main function for dissect a realtime acquisition
 *
 *
 * Xplico - Internet Traffic Decoder
 * By Gianluca Costa <g.costa@iserm.com>
 * Copyright 2007 Gianluca Costa & Andrea de Franceschi. Web: www.xplico.org
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <pcap.h>
#include <string.h>
#include <dirent.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <linux/if.h>
#include "log.h"
#include "packet.h"
#include "dmemory.h"
#include "proto.h"
#include "flow.h"
#include "rltm.h"
#include "report.h"

static int pcap_prot_id;
static unsigned long pkt_serial = 0;


static int RltmParam(int argc, char *argv[], char *intf, char *filter)
{
    int c;
    short n;
    extern char *optarg;
    extern int optind, optopt;

    n = 0;
    while ((c = getopt(argc, argv, "i:f:")) != -1) {
        switch(c) {
        case 'i':
            strcpy(intf, optarg);
            n++;
            break;

        case 'f':
            strcpy(filter, optarg);
            break;

        case '?':
            printf("Error: unrecognized option: -%c\n", optopt);
            return -1;
        }
    }

    if (n != 1)
        return -1;

    return 0;
}


static void RltmDissector(u_char *user, const struct pcap_pkthdr *h, const u_char *bytes)
{
    struct pcap_ref *ref = (struct pcap_ref *)user;
    packet *pkt;
    static time_t tm = 0;
    unsigned long len;

    pkt = PktNew();

    ref->cnt++;
    pkt->raw = DMemMalloc(h->caplen+sizeof(unsigned long)*2+sizeof(char *)+4);
    memcpy(pkt->raw, bytes, h->caplen);
    pkt->raw_len = h->caplen;
    /* align 4b */
    len = pkt->raw_len;
    len = len + 4 - (len%4);
    *((unsigned long *)&(pkt->raw[len])) = ref->dlt;
    *((unsigned long *)&(pkt->raw[len+sizeof(unsigned long)])) = ref->cnt;
    *((char **)(&(pkt->raw[len+sizeof(unsigned long)*2]))) = ref->dev;
    pkt->cap_sec = h->ts.tv_sec;
    pkt->cap_usec = h->ts.tv_usec;
    pkt->serial = pkt_serial;
    FlowSetGblTime(h->ts.tv_sec);

    ProtDissec(pcap_prot_id, pkt);

    /* next serial number */
    pkt_serial++;

    if (time(NULL) > tm) {
        tm = time(NULL) + 30;
        ReportSplash();
    }
}


char* CaptDisOptions(void)
{
    return "{-i <interface>  [-f <filter>]}";
}


void CaptDisOptionsHelp(void)
{
    printf("\t-i interface: eth0, eth1, ...\n");
    printf("\t-f filter\n");
}



int CaptDisMain(int argc, char *argv[])
{
    char errbuf[PCAP_ERRBUF_SIZE];
    char intrf[RLTM_PATH_DIM], filter_app[RLTM_PATH_DIM];
    struct bpf_program filter;     /* The compiled filter */
    pcap_t *cap = NULL;
    int ret;
    struct pcap_ref ref;
    
    /* pcapfile  protocol id */
    pcap_prot_id = ProtId("pcapf");

    /* serial number of packet */
    pkt_serial = 1;

    /* interace & filter */
    intrf[0] = '\0';
    filter_app[0] = '\0';
    ret = RltmParam(argc, argv, intrf, filter_app);
    if (ret != 0) {
        return -1;
    }

    errbuf[sizeof(errbuf) - 1] = '\0';
    errbuf[0] = '\0';
    
    /* open dvice in promiscuous mode */
    cap = pcap_open_live(intrf, BUFSIZ, 1, 0, errbuf);
    if (cap == NULL) {
        printf("Inteface %s: %s\n", intrf, errbuf);
        return -1;
    }
    else {
        /* Compile and apply the filter */
        if (pcap_compile(cap, &filter, filter_app, 1, 0) < 0) {
            printf("Bad filter %s\n", filter_app);
            return -1;
        }
            
        pcap_setfilter(cap, &filter);

        /* interface */
        ref.dev = intrf;
        
        /* data link type */
        ref.dlt = pcap_datalink(cap);
        
        /* packet counter */
        ref.cnt = 0;
        
        /* let pcap loop over the input, passing data to the decryptor */
        pcap_loop(cap, -1, (pcap_handler)&RltmDissector, (u_char*)&ref);
        
        pcap_close(cap);
    }

    return 0;
}
